/*
 * Author:  František Dvořák <valtri@users.sourceforge.net>
 * License: public domain
 *
 * $Id: generator.c $
 */

#include <stdlib.h>

/* 
 * generátor diskrétního rozdělení 
 */
double gen_dist(size_t n, double *x, double *p) {
	double f, y;
	size_t i;

	f = 0;
	y = (double)rand() / RAND_MAX;
	for (i = 0; i < n; i++) {
		if (y < (f += p[i])) return x[i];
	}

	return x[n];
}


/*
 * generování intervalu náhodných čísel v rozsahu 'min' .. 'max'
 */
void random_x(double min, double max, size_t n, double *x) {
	size_t i, j, min_index;
	double range, tmp, min_value;
	
	/* generování čísel */
	range = max - min;
	for (i = 0; i < n; i++) x[i] = (double)rand() / RAND_MAX * range + min;

	/* seřazení, horší než O(n^2) to nebude */
	for (i = 0; i + 1 < n; i++) {
		min_value = x[i];
		min_index = i;
		for (j = i + 1; j < n; j++) {
			if (x[j] < min_value) {
				min_value = x[j];
				min_index = j;
			}
		}
		tmp = x[i];
		x[i] = x[min_index];
		x[min_index] = tmp;
	}
}


/*
 * vygenerování náhodné diskrétní pravděpodobnostní funkce
 */
void random_p(size_t n, double *p) {
	size_t i;
	double avg;

	/* generování pravděpdobností */
	avg = 0;
	for (i = 0; i < n; i++) avg += (p[i] = rand());

	/* normalizace */	
	for (i = 0; i < n; i++) p[i] /= avg;
}


/*
 * generování pole dat o velikosti 'out' s daným diskrétním 
 * pravděpodobnostním rozložením
 */
void random_y(size_t in, double *x, double *p, size_t out, double *y) {
	size_t i;

	for (i = 0; i < out; i++) y[i] = gen_dist(in, x, p);
}
