/*
 * Author:  František Dvořák <valtri@users.sourceforge.net>
 * License: public domain
 *
 * $Id: main.c $
 */

#include <stdio.h>
#include <stdlib.h>

#include "generator.h"
#include "statistics.h"

#define RANGE    60
#define OUTPUT   1000
#define TESTS    5

/* rozsahy pro histogram */
#define COLUMNS 70
#define ROWS    10

#define HEADER "\
+----------+-----------------------------+-----------------------------+\n\
|          |       střední hodnota       |     směrodatná odchylka     |\n\
|          +--------------+--------------+--------------+--------------+\n\
|          |  předpoklad  |  experiment  |  předpoklad  |  experiment  |\n\
+----------+--------------+--------------+--------------+--------------+\n\
"

#define ROW_FORMAT "\
|   %2d.    | %9.2f    | %9.2f    | %9.2f    | %9.2f    |\n\
"

#define FOOTER "\
+----------+--------------+--------------+--------------+--------------+\n\
"


int main() {
	double x[TESTS][RANGE], p[TESTS][RANGE], y[TESTS][OUTPUT];
	double exp_avg[TESTS], exp_aber[TESTS];
	double real_avg[TESTS], real_aber[TESTS];
	int i;
	histogram_t histogram;

	for (i = 0; i < TESTS; i++) {
		/* rozsah proměnné */
		random_x(-10, 100, RANGE, x[i]);
		
		/* pravděpodobnostní rozložení */
		random_p(RANGE, p[i]);
		
		/* výsledná tabulka hodnot */
		random_y(RANGE, x[i], p[i], OUTPUT, y[i]);

		/* předpokládaný výsledek */
		stat_expected(RANGE, x[i], p[i], &exp_avg[i], &exp_aber[i]);

		/* zpracování výsledků */
		stat_real(OUTPUT, y[i], &real_avg[i], &real_aber[i]);
	}
	
	printf("Výsledky:\n\n");
	printf(HEADER);
	for (i = 0; i < TESTS; i++) {	
		/* tisk */
		printf(ROW_FORMAT, i + 1, exp_avg[i], real_avg[i], real_aber[i], real_aber[i]);
	}
	printf(FOOTER);
	printf("\n\n");
	getchar();

	for (i = 0; i < TESTS; i++) {
		printf("%2d. množina:\n", i + 1);
		printf("------------\n\n");
		/* příprava tisku histogramu */
		histogram_init(&histogram, x[i], RANGE, COLUMNS, ROWS);
		/* tisk předpokládaného histogramu */
		printf("  Předpokládaný histogram:\n\n");
		histogram_prep_expected(&histogram, x[i], p[i], RANGE);
		histogram_render(&histogram);
		/* tisk skutečného histogramu */
		printf("  Reálný histogram:\n\n");
		histogram_prep_output(&histogram, y[i], OUTPUT);
		histogram_render(&histogram);
		/* hotovo */
		histogram_free(&histogram);
		printf("\n");
		getchar();
	}
	
	return 0;
}
