/*
 * Author:  František Dvořák <valtri@users.sourceforge.net>
 * License: public domain
 *
 * $Id: statistics.h $
 */

#ifndef STATISTICS_H_INCLUDED
#define STATISTICS_H_INCLUDED

typedef struct {
	double x1, x2;
} interval_t;

typedef struct {
	interval_t *intervals;
	int width, height;
	int *counters;
} histogram_t;

/*
 * vrátí základní info o daném rozdělení
 */
void stat_expected(size_t n, double *x, double *p, double *avg, double *aber);

/*
 * vrátí info z výsledků
 */
void stat_real(size_t n, double *y, double *avg, double *aber);

/*
 * příprava na vypsání histogramu (zpracuje se x-ová osa)
 */
void histogram_init(histogram_t *histogram, double *x, size_t n_p, int width, int height);

/*
 * připraví histogram očekávaných hodnot
 */
void histogram_prep_expected(histogram_t *histogram, double *x, double *p, size_t n_p);

/*
 * připraví histogram experimentálních hodnot
 */
void histogram_prep_output(histogram_t *histogram, double *y, size_t n_y);

/*
 * vypsání histogramu
 */
void histogram_render(histogram_t *histogram);

/*
 * uvolní připravený histogram z paměti
 */
void histogram_free(histogram_t *histogram);

#endif
